#!/usr/bin/env python3

import json
import functools
import traceback

from jaeger_client import Config
from opentracing import Format, logs, tags
from opentracing_instrumentation import get_current_span
from opentracing_instrumentation import client_hooks

TRACER = {}

def init_tracer(service_name):
    global TRACER
    jaeger_cfg = Config(
        config={
            'sampler': {
                'type': 'const',
                'param': 1,
            },
            'logging': True,
        },
        service_name=service_name,
        validate=True
    )

    if not TRACER:
        TRACER = jaeger_cfg.initialize_tracer()
    client_hooks.install_all_patches()


def record_span(_func=None, *, name=None, extract_ctx=False, provide_span=False):
    def decorator_record_span(func):
        @functools.wraps(func)
        def wrapper(*args, **kwargs):
            span_name = name or func.__name__
            if extract_ctx:
                scope = new_scope_from_context(args, span_name)
            else:
                scope = TRACER.start_active_span(span_name)

            try:
                if provide_span:
                    value = func(*args, **kwargs, span=scope.span)
                else:
                    value = func(*args, **kwargs)
            except Exception as ex:
                log_exception(scope.span, ex)
                raise
            finally:
                scope.close()
            return value
        return wrapper

    if _func:
        return decorator_record_span(_func)

    return decorator_record_span


def log_exception(span, exception):
    span.set_tag(tags.ERROR, True)
    span.log_kv({
        logs.EVENT: tags.ERROR,
        logs.MESSAGE: str(exception),
        logs.ERROR_OBJECT: exception,
        logs.ERROR_KIND: type(exception),
        logs.STACK: traceback.format_exc(),
    })


def new_scope_from_context(args, name):
    try:
        msg = json.loads(args[3])
        if 'context' not in msg:
            raise TypeError()
        span_ctx = TRACER.extract(format=Format.TEXT_MAP,
                                  carrier=msg['context'])
        scope = TRACER.start_active_span(name, child_of=span_ctx)
    except (json.decoder.JSONDecodeError, IndexError, TypeError):
        scope = TRACER.start_active_span(name)

    return scope


def context(span):
    ctx = {}
    TRACER.inject(
        span_context=span.context,
        format=Format.TEXT_MAP,
        carrier=ctx
    )
    return ctx


def create_span(name):
    return TRACER.start_span(
        operation_name=name,
        child_of=get_current_span()
    )
