#
# A tool for encrypting/decrypting as used by qstack crypto
#

import base64
import hashlib
import os

from Crypto.Cipher import AES

from .padder import Padder
from .version import Version

VERSION = Version.latest_version()
SEPARATOR = ":"


class SecretsHandler:

    def __init__(self, block_size=16):
        self.block_size = block_size

    def get_derived_key(self, passphrase, salt, iterations):
        length = VERSION.bits/8
        return hashlib.pbkdf2_hmac('sha1', passphrase.encode(),
                                   salt, iterations, length)

    def decrypt(self, passphrase, text):
        (version, salt, iv, enc_text) = self.parse(text)

        key = self.get_derived_key(passphrase, salt, VERSION.iterations)
        mode = "MODE_" + VERSION.mode
        algorithm = eval(VERSION.algorithm)  # will never be user input
        crypter = algorithm.new(key, getattr(algorithm, mode), iv)
        text = crypter.decrypt(enc_text)

        padder = Padder(self.block_size)
        trimmed_text = padder.trim(text)

        return trimmed_text

    def encrypt(self, passphrase, text):
        salt = os.urandom(8)

        padder = Padder(self.block_size)
        padding_method = getattr(padder, VERSION.padding)
        text = padding_method(text)

        key = self.get_derived_key(passphrase, salt, VERSION.iterations)
        iv = os.urandom(16)
        mode = "MODE_" + VERSION.mode
        algorithm = eval(VERSION.algorithm)  # will never be user input
        crypter = algorithm.new(key, getattr(algorithm, mode), iv)
        enc_text = crypter.encrypt(text)

        salt_base64 = base64.b64encode(salt).decode('utf-8')
        iv_base64 = base64.b64encode(iv).decode('utf-8')
        enc_text_base64 = base64.b64encode(enc_text).decode('utf-8')

        return "%s%s%s%s%s%s%s" % (
                VERSION.name,
                SEPARATOR,
                salt_base64,
                SEPARATOR,
                iv_base64,
                SEPARATOR,
                enc_text_base64
        )

    def parse(self, text):
        components = text.split(SEPARATOR)
        if (len(components) != 4):
            raise ValueError('Input has incorrect number of fields')

        version = components[0]
        if version not in [name for name, member in Version.__members__.items()]:
            raise ValueError('Unsupported version')

        try:
            salt = base64.b64decode(components[1])
            iv = base64.b64decode(components[2])
            enc_text = base64.b64decode(components[3])

            return (version, salt, iv, enc_text)
        except Exception as e:
            print("Unable to parse secret format: %s" % text)
            raise
